/************************************************************************
	Copyright 2015 Juan Manuel Rodríguez
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, version 3.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*************************************************************************/
import java.io.BufferedReader;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.NetworkInterface;
import java.net.Proxy;
import java.net.SocketException;
import java.net.URL;
import java.net.URLConnection;
import java.nio.charset.StandardCharsets;
import java.security.cert.X509Certificate;
import java.util.Base64;
import java.util.Enumeration;
import java.util.Properties;

import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLHandshakeException;
import javax.net.ssl.SSLSession;


public class TestSSL {

	static Proxy proxy = null;
	static String proxy_user = "";
	static String proxy_pass = "";
	static String proxy_host = "";
	static String proxy_port = "";
	
	public static void main(String[] args) {

		Properties props = new Properties();

		try {
			InputStream file = new FileInputStream(new File(
					System.getProperty("user.dir") + "/TestSSL.properties"));
			props.load(file);
		} catch (Throwable t) {
			t.printStackTrace();
			return;
		}
		String urlString = "";

		String kspass = props.getProperty("keyStorePass");
		String tspass = props.getProperty("trustStorePass");
		String ksfile = props.getProperty("keyStore");
		String kstipo = props.getProperty("keyStoreType");
		String tsfile = props.getProperty("trustStore");
		proxy_host = props.getProperty("proxy_host");
		proxy_port = props.getProperty("proxy_port");
		proxy_user = props.getProperty("proxy_user");
		proxy_pass = props.getProperty("proxy_pass");

		System.setProperty("javax.net.debug", "all");
		System.setProperty("javax.net.ssl.trustStore", tsfile);
		System.setProperty("javax.net.ssl.trustStorePassword", tspass);
		System.setProperty("javax.net.ssl.keyStore", ksfile);
		System.setProperty("javax.net.ssl.keyStorePassword", kspass);
		System.setProperty("javax.net.ssl.keyStoreType", kstipo);

		System.out.println("Información del key store");
		System.out.println("-------------------------");

		try {
			test(ksfile, kspass, kstipo);
		} catch (Exception e) {
			System.out.println("No se ha podido leer el almacén " + ksfile
					+ ":");
			System.out.println("Exception: " + e);
			return;
		}

		System.out.println("Información del trust store");
		System.out.println("---------------------------");

		try {
			test(tsfile, tspass, "JKS");
		} catch (Exception e) {
			System.out.println("No se ha podido leer el almacén " + tsfile
					+ ":");
			System.out.println("Exception: " + e);
		}

		urlString = props.getProperty("url");
		System.out.println("---------------------------");
		System.out.println("Conectando a " + urlString + "...");
		
		System.out.println ("IP de origen: " + getCurrentEnvironmentNetworkIp());


		if (proxy_host.equals("")) {
			System.out.println("Conexión directa (sin proxy).");
		} else {
			if (proxy_port.equals("")) {
				proxy_port = "80";
			}
			System.out.println("Proxy: " + proxy_host + ":" + proxy_port);
			proxy = new Proxy(Proxy.Type.HTTP, new InetSocketAddress(proxy_host, Integer.parseInt(proxy_port)));
		}

		try {
			InputStream ins = Conexion(urlString);
			BufferedReader in = new BufferedReader(new InputStreamReader(ins));
			String curline;
			while ((curline = in.readLine()) != null) {
				System.out.println(curline);
			}
			ins.close();
		} catch (Throwable t) {
			t.printStackTrace();
		}
	}

	private static InputStream Conexion(String urlstr) throws Exception {
		URLConnection cnx = conexion(urlstr);
		try {
			cnx.connect();
		} 
		catch (SSLHandshakeException e) {
			// Se deshabilita la comprobación de nombre de host y se reintenta la conexión
			System.out.println("---------------------------");
			System.out.println("Error SSLHandshake: se intenta de nuevo sin comprobar el hostname");
			System.out.println("---------------------------");
			HttpsURLConnection connection = (HttpsURLConnection) cnx;
			connection.setHostnameVerifier(new CustomizedHostnameVerifier());
			return connection.getInputStream();
		}
		return cnx.getInputStream();
	}

	private static URLConnection conexion(String urlstr) throws Exception {
		URL url = new URL(urlstr);
		URLConnection cnx = null;
		if (proxy == null) {
			cnx = url.openConnection();
		} else {
			cnx = url.openConnection(proxy);
			if (proxy_user.equals("") && proxy_pass.equals("")) {
				String password = proxy_user + ":" + proxy_pass;
				String encodedPassword = encode(password);
				cnx.setRequestProperty("Proxy-Authorization", encodedPassword);
			}
		}
		return cnx;
	}

	private static String encode(String b) throws Exception {
		return Base64.getEncoder().encodeToString(b.getBytes(StandardCharsets.UTF_8));
	}

	private static void test(String input, String passwd, String tipo) throws Exception {
		java.security.KeyStore ks = java.security.KeyStore.getInstance(tipo);
		ks.load(new java.io.FileInputStream(input), passwd.toCharArray());
		X509Certificate cert;
		boolean[] usos;
		for (Enumeration<String> en = ks.aliases(); en.hasMoreElements();) {
			String alias = (String) en.nextElement();
			System.out.println("ALIAS:" + alias);
			java.security.cert.Certificate[] certs;

			if (ks.isKeyEntry(alias)) {
				certs = ks.getCertificateChain(alias);
			} else {
				certs = new java.security.cert.Certificate[1];
				certs[0] = ks.getCertificate(alias);
			}

			for (int i = 0; i < certs.length; i++) {
				cert = (X509Certificate) certs[i];
				if (ks.isKeyEntry(alias))
					System.out.println(" Certificado "
							+ new Integer(i).toString());
				System.out.println("   Emisor          : "
						+ cert.getIssuerX500Principal().getName());
				System.out.println("   Asunto          : "
						+ cert.getSubjectX500Principal().getName());
				System.out.println("   No antes de     : "
						+ cert.getNotBefore().toString());
				System.out.println("   No después de   : "
						+ cert.getNotAfter().toString());
				/****************************
				 * Key usage extension KeyUsage ::= BIT STRING {
				 * digitalSignature (0), nonRepudiation (1), keyEncipherment
				 * (2), dataEncipherment (3), keyAgreement (4), keyCertSign (5),
				 * cRLSign (6), encipherOnly (7), decipherOnly (8) } Digital
				 * signature Use when the public key is used with a digital
				 * signature mechanism to support security services other than
				 * non-repudiation, certificate signing, or CRL signing. A
				 * digital signature is often used for entity authentication and
				 * data origin authentication with integrity. Non-repudiation
				 * Use when the public key is used to verify digital signatures
				 * used to provide a non-repudiation service. Non-repudiation
				 * protects against the signing entity falsely denying some
				 * action (excluding certificate or CRL signing). Key
				 * encipherment Use when a certificate will be used with a
				 * protocol that encrypts keys. An example is S/MIME enveloping,
				 * where a fast (symmetric) key is encrypted with the public key
				 * from the certificate. SSL protocol also performs key
				 * encipherment. Data encipherment Use when the public key is
				 * used for encrypting user data, other than cryptographic keys.
				 * Key agreement Use when the sender and receiver of the public
				 * key need to derive the key without using encryption. This key
				 * can then can be used to encrypt messages between the sender
				 * and receiver. Key agreement is typically used with
				 * Diffie-Hellman ciphers. Certificate signing Use when the
				 * subject public key is used to verify a signature on
				 * certificates. This extension can be used only in CA
				 * certificates. CRL signing Use when the subject public key is
				 * to verify a signature on revocation information, such as a
				 * CRL. Encipher only Use only when key agreement is also
				 * enabled. This enables the public key to be used only for
				 * enciphering data while performing key agreement. Decipher
				 * only Use only when key agreement is also enabled. This
				 * enables the public key to be used only for deciphering data
				 * while performing key agreement.
				 */
				if (ks.isKeyEntry(alias)) {
					usos = cert.getKeyUsage();
					System.out.println("   Usos :");
					System.out.println("       Identificación       :   "
							+ usos[0]);
					System.out.println("       No repudio           :   "
							+ usos[1]);
					System.out.println("       Cifrado de claves    :   "
							+ usos[2]);
					System.out.println("       Cifrado de datos     :   "
							+ usos[3]);
					System.out.println("       Intercambio de PK    :   "
							+ usos[4]);
					System.out.println("       Firma de certificado :   "
							+ usos[5]);
					System.out.println("       PK para CRL          :   "
							+ usos[6]);
					System.out.println("       PK sólo cifrado      :   "
							+ usos[7]);
					System.out.println("       PK sólo descifrado   :   "
							+ usos[8]);
					System.out.println();
				}
			}
		}
	}

	public static String getCurrentEnvironmentNetworkIp() {
		String currentHostIpAddress = null;
		Enumeration<NetworkInterface> netInterfaces = null;
		try {
			netInterfaces = NetworkInterface.getNetworkInterfaces();

			while (netInterfaces.hasMoreElements()) {
				NetworkInterface ni = netInterfaces.nextElement();
				Enumeration<InetAddress> address = ni.getInetAddresses();
				while (address.hasMoreElements()) {
					InetAddress addr = address.nextElement();
					if (!addr.isLoopbackAddress() && addr.isSiteLocalAddress() && !(addr.getHostAddress().indexOf(":") > -1)) {
						currentHostIpAddress = addr.getHostAddress();
					}
				}
			}
			if (currentHostIpAddress == null) {
				currentHostIpAddress = "127.0.0.1";
			}

		} catch (SocketException e) {
			currentHostIpAddress = "127.0.0.1";
		}

		return currentHostIpAddress;
	}

	private static class CustomizedHostnameVerifier implements HostnameVerifier {
		public boolean verify(String hostname, SSLSession session) {
			return true;
		}
	}

}

